@@ -93,7 +93,11 @@ class TestApp(object):
         if extra_environ:
             environ.update(extra_environ)
         return environ
-
+    
+    def _remove_fragment(self, url):
+        scheme, netloc, path, query, fragment = urlparse.urlsplit(url)
+        return urlparse.urlunsplit((scheme, netloc, path, query, ""))
+    
     def get(self, url, params=None, headers=None, extra_environ=None,
             status=None, expect_errors=False):
         """
@@ -141,6 +145,7 @@ class TestApp(object):
             url, environ['QUERY_STRING'] = url.split('?', 1)
         else:
             environ['QUERY_STRING'] = ''
+        url = self._remove_fragment(url)
         req = TestRequest.blank(url, environ)
         if headers:
             req.headers.update(headers)
@@ -175,6 +180,7 @@ class TestApp(object):
         environ['CONTENT_LENGTH'] = str(len(params))
         environ['REQUEST_METHOD'] = method
         environ['wsgi.input'] = StringIO(params)
+        url = self._remove_fragment(url)
         req = TestRequest.blank(url, environ)
         if headers:
             req.headers.update(headers)
@@ -205,13 +211,11 @@ class TestApp(object):
             status=None, upload_files=None, expect_errors=False,
             content_type=None):
         """
-        Do a PUT request.  Very like the ``.get()`` method.
-        ``params`` are put in the body of the request.
-
-        ``upload_files`` is for file uploads.  It should be a list of
-        ``[(fieldname, filename, file_content)]``.  You can also use
-        just ``[(fieldname, filename)]`` and the file content will be
-        read from disk.
+        Do a PUT request.  Very like the ``.put()`` method.
+        ``params`` are put in the body of the request, if params is a
+        tuple, dictionary, list, or iterator it will be urlencoded and
+        placed in the body as with a POST, if it is string it will not
+        be encoded, but placed in the body directly.
 
         Returns a ``webob.Response`` object.
         """
@@ -725,7 +729,7 @@ class TestResponse(Response):
                 print >> sys.stderr, "Actual response (has %r)" % no_s
                 print >> sys.stderr, self
                 raise IndexError(
-                    "Body contains string %r" % s)
+                    "Body contains bad string %r" % no_s)
 
     def __str__(self):
         simple_body = '\n'.join([l for l in self.body.splitlines()
@@ -1170,6 +1174,9 @@ class Field(object):
 
     value = property(value__get, value__set)
 
+class NoValue(object):
+    pass
+
 class Select(Field):
 
     """
@@ -1181,8 +1188,15 @@ class Select(Field):
         self.options = []
         # Undetermined yet:
         self.selectedIndex = None
-
+        # we have no forced value
+        self._forced_value = NoValue
+    
+    def force_value(self, value):
+        self._forced_value = value
+    
     def value__set(self, value):
+        if self._forced_value is not NoValue:
+            self._forced_value = NoValue
         for i, (option, checked) in enumerate(self.options):
             if option == str(value):
                 self.selectedIndex = i
@@ -1194,7 +1208,9 @@ class Select(Field):
                 [repr(o) for o, c in self.options])))
 
     def value__get(self):
-        if self.selectedIndex is not None:
+        if self._forced_value is not NoValue:
+            return self._forced_value
+        elif self.selectedIndex is not None:
             return self.options[self.selectedIndex][0]
         else:
             for option, checked in self.options:
@@ -1221,7 +1237,12 @@ class MultipleSelect(Field):
         self.options = []
         # Undetermined yet:
         self.selectedIndices = []
-
+        self._forced_values = []
+    
+    def force_value(self, values):
+        self._forced_values = values
+        self.selectedIndices = []
+    
     def value__set(self, values):
         str_values = [str(value) for value in values]
         self.selectedIndicies = []
@@ -1237,18 +1258,20 @@ class MultipleSelect(Field):
                         [repr(o) for o, c in self.options])))
 
     def value__get(self):
+        selected_values = []
         if self.selectedIndices:
-            return [self.options[i][0] for i in self.selectedIndices]
-        else:
+            selected_values = [self.options[i][0] for i in self.selectedIndices]
+        elif not self._forced_values:
             selected_values = []
             for option, checked in self.options:
                 if checked:
                     selected_values.append(option)
-                
-            if self.options and (not selected_values):
-                selected_values = None
-            return selected_values
-    
+        if self._forced_values:
+            selected_values += self._forced_values
+        
+        if self.options and (not selected_values):
+            selected_values = None
+        return selected_values
     value = property(value__get, value__set)
 
 Field.classes['multiple_select'] = MultipleSelect
@@ -1259,6 +1282,19 @@ class Radio(Select):
     Field representing ``<input type="radio">``
     """
 
+    def value__get(self):
+        if self.selectedIndex is not None:
+            return self.options[self.selectedIndex][0]
+        else:
+            for option, checked in self.options:
+                if checked:
+                    return option
+            else:
+                return None
+
+    value = property(value__get, Select.value__set)
+
+
 Field.classes['radio'] = Radio
 
 class Checkbox(Field):
@@ -1292,8 +1328,31 @@ class Text(Field):
     Field representing ``<input type="text">``
     """
 
+    def value__get(self):
+        if self._value is None:
+            return ''
+        else:
+            return self._value
+
+    value = property(value__get, Field.value__set)
+
 Field.classes['text'] = Text
 
+
+class File(Field):
+    """
+    Field representing ``<input type="file">``
+    """
+
+    ## FIXME: This doesn't actually handle file uploads and enctype
+    def value__get(self):
+        if self._value is None:
+            return ''
+        else:
+            return self._value
+
+Field.classes['file'] = File
+
 class Textarea(Text):
     """
     Field representing ``<textarea>``
